@@ -1,3 +1,18 @@
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 from __future__ import division
 import uuid
 import math
@@ -60,6 +75,7 @@ class BoxValidator:
 		self.startOffset = startOffset
 		self.returnOffset = None
 		self.isValid = None
+		self.bTypeString = bType
 
 	def validate(self):
 		try:
@@ -212,8 +228,27 @@ class BoxValidator:
 	# Validator functions for boxes
 	
 	def validate_unknownBox(self):
-		# Unknown box: print warning message to screen
+
+		# Although jpylyzer doesn't "know" anything about this box, we
+		# can at least report the 4 characters from the Box Type field
+		# (TBox) here
 		
+		boxType=self.bTypeString
+		
+		# If boxType contains any device control characters (e.g. because of
+		# file corruption), replace them  with printable character
+		if bc.containsControlCharacters(boxType):
+			boxType=bc.replaceControlCharacters(boxType)
+		
+		# Decode to string with Latin encoding
+		# Elementtree will deal with any non-ASCII characters by replacing
+		# them with numeric entity references
+		boxType=boxType.decode("iso-8859-15","strict")
+		
+		# Add (cleaned up) boxType string to output
+		self.addCharacteristic( "boxType", boxType)
+		
+		# Print warning message to screen		
 		printWarning("ignoring unknown box")
 
 	def validate_signatureBox(self):
@@ -222,7 +257,7 @@ class BoxValidator:
 		# Check box size, which should be 4 bytes
 		self.testFor("boxLengthIsValid", len(self.boxContents) == 4)
 		
-		# Signature (*not* added to characteristics output, because it contains non-printable characters)
+		# Signature *not* added to characteristics output, because it contains non-printable characters)
 		self.testFor("signatureIsValid", self.boxContents[0:4] == b'\x0d\x0a\x87\x0a')
 
 	def validate_fileTypeBox(self):
@@ -511,7 +546,7 @@ class BoxValidator:
 				tagSignatures.append(tagSignatureElements[i].text)
 
 			# Step 3: verify non-existence of "AToB0Tag"
-			self.testFor("iccNoLUTBasedProfile", b'AToB0Tag' not in tagSignatures)
+			self.testFor("iccNoLUTBasedProfile", b'A2B0' not in tagSignatures)
 
 		elif meth == 3:
 			# ICC profile embedded using "Any ICC" method. Belongs to Part 2 of the
@@ -1022,7 +1057,7 @@ class BoxValidator:
 		marker,segLength,segContents,offsetNext=self._getMarkerSegment(offset)
 		foundSIZMarker = (marker == b'\xff\x51')
 		self.testFor("foundSIZMarker", foundSIZMarker)
-
+		
 		if foundSIZMarker:
 			# Validate SIZ segment
 			resultSIZ, characteristicsSIZ = BoxValidator(marker, segContents).validate() # validateSIZ(segContents)
@@ -1048,7 +1083,7 @@ class BoxValidator:
 
 		while marker != b'\xff\x90' and offsetNext !=-9999:
 			marker,segLength,segContents,offsetNext=self._getMarkerSegment(offset)
-
+			
 			if marker == b'\xff\x52':
 				# COD (coding style default) marker segment
 				# COD is required
@@ -1116,7 +1151,7 @@ class BoxValidator:
 
 		# Remainder of codestream is a sequence of tile parts, followed by one
 		# end-of-codestream marker
-		
+				
 		# Expected number of tiles (as calculated from info in SIZ marker)
 		numberOfTilesExpected=self.characteristics.findElementText('siz/numberOfTiles')
 		
@@ -1144,10 +1179,10 @@ class BoxValidator:
 				resultTilePart, characteristicsTilePart,offsetNext = BoxValidator(marker, self.boxContents, offset).validate()
 				# Add analysis results to test results tree
 				tilePartTests.appendIfNotEmpty(resultTilePart)
-
+				
 				# Add extracted characteristics to characteristics tree
 				tilePartCharacteristics.append(characteristicsTilePart)
-				
+								
 				tileIndex=characteristicsTilePart.findElementText('sot/isot')
 				tilePartIndex=characteristicsTilePart.findElementText('sor/tpsot')
 				tilePartsOfTile=characteristicsTilePart.findElementText('sot/tnsot')
@@ -1165,14 +1200,15 @@ class BoxValidator:
 
 				if offsetNext != offset:
 					offset = offsetNext
+				else:
+					# offsetNext same as offset: this happens if image only contains
+					# one single tile-part (psot=0), in which case we break out of
+					# this loop
+					break
 
 		# Length of tileIndices should equal numberOfTilesExpected
 		self.testFor("foundExpectedNumberOfTiles", len(tileIndices) == numberOfTilesExpected)
-		
-		#test = set(tilePartsPerTileExpected.items()) - set(tilePartsPerTileFound.items()) 
-		
-		#print(len(test))
-		
+				
 		# Found numbers of tile	parts per tile should match expected	
 		self.testFor("foundExpectedNumberOfTileParts", len(set(tilePartsPerTileExpected.items()) - set(tilePartsPerTileFound.items())) == 0)
 
@@ -1663,7 +1699,7 @@ class BoxValidator:
 		#   of a tile; there is currently no check on this (may be added later)
 
 		offset = self.startOffset
-
+		
 		# Read first marker segment, which is a  start of tile (SOT) marker segment
 		marker,segLength,segContents,offsetNext=self._getMarkerSegment(offset)
 
@@ -1671,7 +1707,7 @@ class BoxValidator:
 		# tilePartLength is value of psot, which is the total length of this tile
 		# including the SOT marker. Note that psot may be 0 for last tile!
 		resultSOT, characteristicsSOT, tilePartLength = BoxValidator('startOfTile', segContents).validate()
-
+		
 		# Add analysis results to test results tree
 		self.tests.appendIfNotEmpty(resultSOT)
 
@@ -1694,7 +1730,7 @@ class BoxValidator:
 
 		while marker != b'\xff\x93' and offsetNext !=-9999:
 			marker,segLength,segContents,offsetNext=self._getMarkerSegment(offset)
-						
+									
 			if marker==b'\xff\x52':
 				# COD (coding style default) marker segment
 				
@@ -1737,21 +1773,22 @@ class BoxValidator:
 
 				offset=offsetNext
 				
-			elif marker in[b'\xff\x52',b'\xff\x5d',b'\xff\x5e', \
+			elif marker in[b'\xff\x53',b'\xff\x5d',b'\xff\x5e', \
 					b'\xff\x5f',b'\xff\x61',b'\xff\x58']:
 				# COC, QCC, RGN, POC, PPT or PLT marker: ignore and
 				# move on to next one
+				# Bugfix 1.5.2: COC marker was previously missing (changed x52 to x53!)
 				offset=offsetNext
 
 			else:
 				# Unknown marker segment: ignore and move on to next one
 				offset=offsetNext
-			
-		# Last marker segment (at self.startOffset + 12) should be start-
-		# of-data (SOD) marker
-		marker,segLength,segContents,offsetNext=self._getMarkerSegment(self.startOffset + 12)
-		
+				
+		# Last marker segment should be start-of-data (SOD) marker
 		self.testFor("foundSODMarker",marker == b'\xff\x93')
+		
+		# Bugfix 1.5.2: previous versions mistakenly assumed SOD at self.startOffset + 12!
+		# Goes wrong if til part contains any optional markers. Fixed now!
 					
 		# Position of first byte in next tile
 		offsetNextTilePart = self.startOffset + tilePartLength
